/******************************************************************************/
/*                Start of Data Sync Device Object Definition                 */
/******************************************************************************/

// JavaScript Object to hold Device information for webservice calls
var Device = xMattersWS.extend({

  LOG_SOURCE: "dataSyncDevice.js: ",

  EMPTY_RESULT_SET: "EMPTY_RESULT_SET",
  UNKNOWN_DEVICE: "UNKNOWN_DEVICE",
  PHONE_REGEX: /(\d{3})(\d{7})(\d*)$/,
  
  /**
   * Initialize the class variables
   * @param deviceType identifies the device type that is being processed.
   */
  init: function(deviceType)
  {
    this._super();
    this.deviceType = deviceType;
  
    this.owner = "";
    this.active = true;
    this.defaultDevice = false;
    this.delay = 0;
    this.externallyOwned = false;
    this.deviceName = "";
    this.priorityThreshold = "LOW";
    this.identifier = "1";
    this.name = "";
    this.deviceIdentifier = "";
  },
  
  /**
   * Creates an Add Request for the device
   * @return instance of the SOAPMessage populated to send to the webservice
   */
  add: function()
  {
    var msg = new SOAPMessage(this.XMATTERS_WSDL_DEFN, "AddDevice", "Add" + this.deviceType + "Device", this.XM_SOAP_VERSION);
    this.initializeAddOrUpdate(msg);
    return msg;
  },

  /**
   * Creates an update request for the device
   * @return instance of the SOAPMessage populated to send to the webservice
   */
  update: function()
  {
    if (!"".equalsIgnoreCase(this.deviceIdentifier))
    {
      var msg = new SOAPMessage(this.XMATTERS_WSDL_DEFN, "UpdateDevice", "Update" + this.deviceType + "Device", this.XM_SOAP_VERSION);
      this.initializeAddOrUpdate(msg);
      msg.setParameter("deviceIdentifier", this.deviceIdentifier);
      return msg;
    }
    return this.UNKNOWN_DEVICE;
  },

  /**
   * Initialize the SOAPMessage with the common attributes for add or update
   * @param msg instance of the SOAPMessage to initialize
   */
  initializeAddOrUpdate: function(msg)
  {
    msg.setParameter("user", this.wsUser);
    msg.setParameter("password", this.wsPassword);
    msg.setParameter("company", this.wsCompany);
    msg.setParameter("owner", this.owner);
    msg.setParameter("active", this.active);
    msg.setParameter("default", this.defaultDevice);
    msg.setParameter("delay", this.delay);
    msg.setParameter("externallyOwned", this.externallyOwned);
    msg.setParameter("deviceName", this.deviceName);
    msg.setParameter("priorityThreshold", this.priorityThreshold);
    msg.setParameter("identifier", this.identifier);
    msg.setParameter("name", this.name);
  },
  
  /**
   * Remove the device from xMatters
   */
  remove: function()
  {
    // Retrieve the device information as the update requires the device id
    if(this.exists())
    {
      var msg = new SOAPMessage(this.XMATTERS_WSDL_DEFN, "DeleteDevice", "DeleteDevice", this.XM_SOAP_VERSION);

      msg.setParameter("user", this.wsUser);
      msg.setParameter("password", this.wsPassword);
      msg.setParameter("company", this.wsCompany);
      msg.setParameter("deviceIdentifier", this.deviceIdentifier);
    
      return this.syncSendReceive(this.endPoint, msg, "Delete Device", this.owner + "|" + this.deviceName).xmns::DeleteDeviceResponse.child(QName(xmns, "return")).xmns::status;
    }
    return this.UNKNOWN_DEVICE;
  },
  
  /**
   * Checks if the device exists in xMatters.
   */
  exists: function()
  {
    var msg = new SOAPMessage(this.XMATTERS_WSDL_DEFN, "FindDevices", "FindDevices", this.XM_SOAP_VERSION);

    msg.setParameter("user", this.wsUser);
    msg.setParameter("password", this.wsPassword);
    msg.setParameter("company", this.wsCompany);
    msg.setParameter("owner", this.owner);
    msg.setParameter("targetName", this.owner + "|" + this.deviceName);
    
    var response = this.sendReceive(this.endPoint, msg);
    var deviceExists = !this.EMPTY_RESULT_SET.equalsIgnoreCase(response.xmns::FindDevicesResponse.child(QName(xmns, "return")).xmns::status);
    
    if (deviceExists)
    {
      var devices = response.xmns::FindDevicesResponse.child(QName(xmns, "return")).xmns::devices.child(QName(xmns, this.deviceType + "Device"));
      this.deviceIdentifier = devices.xmns::deviceIdentifier;
    }
    return deviceExists;
  },
  
  toString: function()
  {
    return "<Device>" + 
           "<deviceType>" + this.deviceType + "</deviceType>" +
           "<deviceName>" + this.deviceName + "</deviceName>" +
           "</Device>";
  }
});

/**
 * Blackberry Device - subclass of Device
 */
var BlackberryDevice = Device.extend({

  init: function()
  {
    this._super("blackberry");
    this.deviceName = "Work BES";
    this.name = "BES";
    this.pinOrEmail = "";
  },
  
  /**
   * Add the Blackberry device to xMatters
   * @return response body from the SOAP Response
   */
  add: function()
  {
    var msg = this._super();
    msg.setParameter("pinOrEmail", this.pinOrEmail);
    return this.syncSendReceive(this.endPoint, msg, "Add Blackberry Device", this.owner + "|" + this.deviceName).xmns::AddDeviceResponse.child(QName(xmns, "return")).xmns::status;
  },

  /**
   * Update the Blackberry device in xMatters
   * @return response body from the SOAP Response
   */
  update: function()
  {
    var msg = this._super();
    msg.setParameter("pinOrEmail", this.pinOrEmail);
    return this.syncSendReceive(this.endPoint, msg, "Update Blackberry Device", this.owner + "|" + this.deviceName).xmns::UpdateDeviceResponse.child(QName(xmns, "return")).xmns::status;
  },
  
  /**
   * Checks that the Blackberry device information has been provided
   */
  isValid: function()
  {
    return !"".equalsIgnoreCase(this.pinOrEmail);
  }
  
});

/**
 * Email Device - subclass of Device
 */
var EmailDevice = Device.extend({

  init: function()
  {
    this._super("email");
    this.deviceName = "Work Email";
    this.name = "SMTP Email";
    this.address = "";
  },
  
  /**
   * Add the email device to xMatters
   * @return response body from the SOAP Response
   */
  add: function()
  {
    var msg = this._super();
    msg.setParameter("address", this.address);
    return this.syncSendReceive(this.endPoint, msg, "Add Email Device", this.owner + "|" + this.deviceName).xmns::AddDeviceResponse.child(QName(xmns, "return")).xmns::status;
  },

  /**
   * Update the email device in xMatters
   * @return response body from the SOAP Response
   */
  update: function()
  {
    var msg = this._super();
    msg.setParameter("address", this.address);
    return this.syncSendReceive(this.endPoint, msg, "Update Email Device", this.owner + "|" + this.deviceName).xmns::UpdateDeviceResponse.child(QName(xmns, "return")).xmns::status;
  },

  /**
   * Checks that the email device information has been provided
   */
  isValid: function()
  {
    return !"".equalsIgnoreCase(this.address);
  }
  
});

/**
 * Generic Device - subclass of Device
 */
var GenericDevice = Device.extend({

  init: function()
  {
    this._super("generic");
    this.pin = "";
  },
  
  /**
   * Add the Generic device to xMatters
   * @return response body from the SOAP Response
   */
  add: function()
  {
    var msg = this._super();
    msg.setParameter("pin", this.pin);
    return this.syncSendReceive(this.endPoint, msg, "Add Generic Device", this.owner + "|" + this.deviceName).xmns::AddDeviceResponse.child(QName(xmns, "return")).xmns::status;
  },

  /**
   * Update the Generic device in xMatters
   * @return response body from the SOAP Response
   */
  update: function()
  {
    var msg = this._super();
    msg.setParameter("pin", this.pin);
    return this.syncSendReceive(this.endPoint, msg, "Update Generic Device", this.owner + "|" + this.deviceName).xmns::UpdateDeviceResponse.child(QName(xmns, "return")).xmns::status;
  },

  /**
   * Checks that the generic device information has been provided
   */
  isValid: function()
  {
    return !"".equalsIgnoreCase(this.pin);
  }
  
});

/**
 * IM Device - subclass of Device
 */
var IMDevice = Device.extend({

  init: function()
  {
    this._super("im");
    this.deviceName = "Instant Messenger";
    this.name = "Jabber COM";
    this.address = "";
  },
  
  /**
   * Add the IM device to xMatters
   * @return response body from the SOAP Response
   */
  add: function()
  {
    var msg = this._super();
    msg.setParameter("address", this.address);
    return this.syncSendReceive(this.endPoint, msg, "Add IM Device", this.owner + "|" + this.deviceName).xmns::AddDeviceResponse.child(QName(xmns, "return")).xmns::status;
  },

  /**
   * Update the IM device in xMatters
   * @return response body from the SOAP Response
   */
  update: function()
  {
    var msg = this._super();
    msg.setParameter("address", this.address);
    return this.syncSendReceive(this.endPoint, msg, "Update IM Device", this.owner + "|" + this.deviceName).xmns::UpdateDeviceResponse.child(QName(xmns, "return")).xmns::status;
  },

  /**
   * Checks that the IM device information has been provided
   */
  isValid: function()
  {
    return !"".equalsIgnoreCase(this.address);
  }
  
});

/**
 * NumericPager Device - subclass of Device
 */
var NumericPagerDevice = Device.extend({

  init: function()
  {
    this._super("numericPager");
    this.deviceName = "Numeric Pager";
    this.name = "SkyPage Numeric Pager";
    this.areaCode = "";
    this.countryCodeOverride = "";
    this.number = "";
    this.pin = "";
  },
  
  /**
   * Add the NumericPager device to xMatters
   * @return response body from the SOAP Response
   */
  add: function()
  {
    var msg = this._super();
    msg.setParameter("areaCode", this.areaCode);
    msg.setParameter("countryCodeOverride", this.countryCodeOverride);
    msg.setParameter("number", this.number);
    msg.setParameter("pin", this.pin);
    return this.syncSendReceive(this.endPoint, msg, "Add NumericPager Device", this.owner + "|" + this.deviceName).xmns::AddDeviceResponse.child(QName(xmns, "return")).xmns::status;
  },

  /**
   * Update the NumericPager device in xMatters
   * @return response body from the SOAP Response
   */
  update: function()
  {
    var msg = this._super();
    msg.setParameter("areaCode", this.areaCode);
    msg.setParameter("countryCodeOverride", this.countryCodeOverride);
    msg.setParameter("number", this.number);
    msg.setParameter("pin", this.pin);
    return this.syncSendReceive(this.endPoint, msg, "Update NumericPager Device", this.owner + "|" + this.deviceName).xmns::UpdateDeviceResponse.child(QName(xmns, "return")).xmns::status;
  },

  /**
   * Checks that the NumericPager device information has been provided
   */
  isValid: function()
  {
    return !"".equalsIgnoreCase(this.number);
  }
  
});

/**
 * TextPager Device - subclass of Device
 */
var TextPagerDevice = Device.extend({

  init: function()
  {
    this._super("textPager");
    this.deviceName = "Pager";
    this.name = "Virtual Text Pager";
    this.pin = "";
    this.twoWay = "false";
  },
  
  /**
   * Add the TextPager device to xMatters
   * @return response body from the SOAP Response
   */
  add: function()
  {
    var msg = this._super();
    msg.setParameter("pin", this.pin);
    msg.setParameter("two_way", this.twoWay);
    return this.syncSendReceive(this.endPoint, msg, "Add TextPager Device", this.owner + "|" + this.deviceName).xmns::AddDeviceResponse.child(QName(xmns, "return")).xmns::status;
  },

  /**
   * Update the TextPager device in xMatters
   * @return response body from the SOAP Response
   */
  update: function()
  {
    var msg = this._super();
    msg.setParameter("pin", this.pin);
    msg.setParameter("two_way", this.twoWay);
    return this.syncSendReceive(this.endPoint, msg, "Update TextPager Device", this.owner + "|" + this.deviceName).xmns::UpdateDeviceResponse.child(QName(xmns, "return")).xmns::status;
  },
  
  /**
   * Checks that the TextPager device information has been provided
   */
  isValid: function()
  {
    return !"".equalsIgnoreCase(this.pin);
  }
    
});

/**
 * TextPhone Device - subclass of Device
 */
var TextPhoneDevice = Device.extend({

  init: function()
  {
    this._super("textPhone");
    this.deviceName = "SMS Phone";
    this.name = "Virtual Text Phone";
    this.number = "";
  },
  
  /**
   * Add the TextPhone device to xMatters
   * @return response body from the SOAP Response
   */
  add: function()
  {
    var msg = this._super();
    msg.setParameter("number", this.number);
    return this.syncSendReceive(this.endPoint, msg, "Add TextPhone Device", this.owner + "|" + this.deviceName).xmns::AddDeviceResponse.child(QName(xmns, "return")).xmns::status;
  },

  /**
   * Update the TextPhone device in xMatters
   * @return response body from the SOAP Response
   */
  update: function()
  {
    var msg = this._super();
    msg.setParameter("number", this.number);
    return this.syncSendReceive(this.endPoint, msg, "Update TextPhone Device", this.owner + "|" + this.deviceName).xmns::UpdateDeviceResponse.child(QName(xmns, "return")).xmns::status;
  },
  
  /**
   * Checks that the TextPhone device information has been provided
   */
  isValid: function()
  {
    return !"".equalsIgnoreCase(this.number);
  }
  
});

/**
 * Voice Device - subclass of Device
 */
var VoiceDevice = Device.extend({

  init: function()
  {
    this._super("voice");
    this.deviceName = "Work Phone"
    this.name = "Phone Engine";
    this.areaCode = "";
    this.countryCodeOverride = "";
    this.extension = "";
    this.number = "";
  },
  
  /**
   * Add the Voice device to xMatters
   * @return response body from the SOAP Response
   */
  add: function()
  {
    var msg = this._super();
    msg.setParameter("areaCode", this.areaCode);
    msg.setParameter("countryCodeOverride", this.countryCodeOverride);
    msg.setParameter("extension", this.extension);
    msg.setParameter("number", this.number);
    return this.syncSendReceive(this.endPoint, msg, "Add Voice Device", this.owner + "|" + this.deviceName).xmns::AddDeviceResponse.child(QName(xmns, "return")).xmns::status;
  },

  /**
   * Update the Voice device in xMatters
   * @return response body from the SOAP Response
   */
  update: function()
  {
    var msg = this._super();
    msg.setParameter("areaCode", this.areaCode);
    msg.setParameter("countryCodeOverride", this.countryCodeOverride);
    msg.setParameter("extension", this.extension);
    msg.setParameter("number", this.number);
    return this.syncSendReceive(this.endPoint, msg, "Update Voice Device", this.owner + "|" + this.deviceName).xmns::UpdateDeviceResponse.child(QName(xmns, "return")).xmns::status;
  },
  
  /**
   * Checks that the Voice device information has been provided
   */
  isValid: function()
  {
    if (this.number != null && !"".equalsIgnoreCase(this.number))
    {
      // Try and parse the phone number into the area code, country code extension and number if possible
      var phoneNo = this.number.toString();
      phoneNo = phoneNo.replace(/[^\d\.]+/g, '');

      if (this.PHONE_REGEX.test(phoneNo)) 
      {
        this.areaCode = phoneNo.replace(this.PHONE_REGEX, "$1");
        this.number = phoneNo.replace(this.PHONE_REGEX, "$2");
        this.extension = phoneNo.replace(this.PHONE_REGEX, "$3");
      }
      else 
      {
        this.areaCode = "";
        this.number = phoneNo;
        this.extension = "";
      }
      return true;
    }
    return false;
  }
  
});
