/******************************************************************************/
/*                Start of Data Sync Group Object Definition                  */
/******************************************************************************/
var Group = xMattersWS.extend({

  LOG_SOURCE: "dataSyncGroup.js: ",
  UNKNOWN_GROUP: "UNKNOWN_GROUP",

  /**
   * Initialize the class variables
   */
  init: function()
  {
    this._super(); 
    this.groupName = "";
    this.name = "";
    this.active = true;
    this.allowDuplicates = true;
    this.description = "";
    this.externallyOwned = false;
    this.observedByAll = true;
    this.observers = [];
    this.site = "Default Site";
    this.supervisors = ["companyadmin"];
    this.timeZone = "US/Pacific";
    this.useDefaultDevices = true;
    this.team = new Team();
    this.coverage = new Coverage();
    
    this.groupExists = null;
  },

  /**
   * Process the observer array and create the observer collection for the webservice call
   * @return XML containing the group observers
   */
  getObservers: function()
  {
    var observersXml = "";
    for (var i=0; i<this.observers.length; i++)
    {
      observersXml += "<sch:observer><sch:name>" + StringEscapeUtils.escapeXml(this.observers[i]) + "</sch:name></sch:observer>";
    }
    return observersXml;
  },
    
  /**
   * Process the supervisor array and create the supervisor collection for the webservice call
   * @return XML containing the group supervisors
   */
  getSupervisors: function()
  {
    var supervisorsXml = "";
    for (var i=0; i<this.supervisors.length; i++)
    {
      supervisorsXml += "<sch:supervisor><sch:targetName>" + StringEscapeUtils.escapeXml(this.supervisors[i]) + "</sch:targetName></sch:supervisor>";
    }
    return supervisorsXml;
  },

  /**
   * Add the group to xMatters
   * @return response body from the SOAP Response
   */
  add: function()
  {
    var msg = new SOAPMessage(this.XMATTERS_WSDL_DEFN, "AddGroup", "AddGroup", this.XM_SOAP_VERSION);
    this.initializeAddOrUpdate(msg);
    
    return this.syncSendReceive(this.endPoint, msg, "Add Group", this.name).xmns::AddGroupResponse.child(QName(xmns, "return")).xmns::status;
  },

  /**
   * Update the group in xMatters
   * @return response body from the SOAP Response
   */
  update: function()
  {
    if (!"".equalsIgnoreCase(this.groupName))
    {
      var msg = new SOAPMessage(this.XMATTERS_WSDL_DEFN, "UpdateGroup", "UpdateGroup", this.XM_SOAP_VERSION);
      this.initializeAddOrUpdate(msg);
      
      msg.setParameter("groupName", this.groupName);

      return this.syncSendReceive(this.endPoint, msg, "Update Group", this.name).xmns::UpdateGroupResponse.child(QName(xmns, "return")).xmns::status;
    }
    return this.UNKNOWN_GROUP;
  },
  
  /**
   * Delete the group from xMatters
   * @return response body from the SOAP Response
   */
  remove: function()
  {
    if (this.exists())
    {
      var msg = new SOAPMessage(this.XMATTERS_WSDL_DEFN, "DeleteGroup", "DeleteGroup", this.XM_SOAP_VERSION);
      msg.setParameter("user", this.wsUser);
      msg.setParameter("password", this.wsPassword);
      msg.setParameter("company", this.wsCompany);
      msg.setParameter("groupName", this.name);

      return this.syncSendReceive(this.endPoint, msg, "Delete Group", this.name).xmns::DeleteGroupResponse.child(QName(xmns, "return")).xmns::status;
    }
    return this.UNKNOWN_GROUP;
  },
  
  /**
   * Private helper function to initialize the values in the SOAP Message for the Group Object
   * @param msg instance of SOAPMessage to populate
   */
  initializeAddOrUpdate: function(msg)
  {
    msg.setParameter("user", this.wsUser);
    msg.setParameter("password", this.wsPassword);
    msg.setParameter("company", this.wsCompany);
    msg.setParameter("active", this.active);
    msg.setParameter("allowDuplicates", this.allowDuplicates);
    msg.setParameter("description", !"".equalsIgnoreCase(this.description) ? this.description : this.name + " Group");
    msg.setParameter("externallyOwned", this.externallyOwned);
    msg.setParameter("name", this.name);
    msg.setParameter("observedByAll", this.observedByAll);
    msg.setParameter("observers", this.getObservers(), false);
    msg.setParameter("site", this.site);
    msg.setParameter("supervisors", this.getSupervisors(), false);
    msg.setParameter("timeZone", this.timeZone);
    msg.setParameter("useDefaultDevices", this.useDefaultDevices);
  },
  
  /**
   * Checks if the group exists in xMatters.
   */
  exists: function()
  {
    if (this.groupExists == null)
    {
      this.query();
    }
    return this.groupExists;
  },

  /**
   * Checks if the group exists in xMatters.
   */
  query: function()
  {
    var msg = new SOAPMessage(this.XMATTERS_WSDL_DEFN, "QueryGroup", "QueryGroup", this.XM_SOAP_VERSION);
    msg.setParameter("user", this.wsUser);
    msg.setParameter("password", this.wsPassword);
    msg.setParameter("company", this.wsCompany);
    msg.setParameter("groupName", this.name);

    var response = this.sendReceive(this.endPoint, msg);
    this.groupExists = !this.UNKNOWN_GROUP.equalsIgnoreCase(response.xmns::QueryGroupResponse.child(QName(xmns, "return")).xmns::status);
    
    if (this.groupExists)
    {
      var group = response.xmns::QueryGroupResponse.child(QName(xmns, "return")).xmns::group;
      this.groupName = group.xmns::name;
    }
    return response;
   },
  
  /**
   * Checks that the minimum group information has been provided
   */
  isValid: function()
  {
    return !"".equalsIgnoreCase(this.name);
  },
  
  /**
   * Returns a XML String representation of the Object
   * @return XML formatted String
   */
  toString: function()
  {
    return "<Group>" + 
           "<groupName>" + this.groupName + "</groupName>" +
           "<active>" + this.active + "</active>" +
           "<allowDuplicates>" + this.allowDuplicates + "</allowDuplicates>" +
           "<description>" + this.description + "</description>" +
           "<externallyOwned>" + this.externallyOwned + "</externallyOwned>" +
           "<name>" + this.name + "</name>" +
           "<observedByAll>" + this.observedByAll + "</observedByAll>" +
           "<observers>" + this.getObservers() + "</observers>" +
           "<site>" + this.site + "</site>" +
           "<supervisors>" + this.getSupervisors() + "</supervisors>" +
           "<timezone>" + this.timeZone + "</timezone>" +
           "<useDefaultDevices>" + this.useDefaultDevices + "</useDefaultDevices>" +
           "</Group>";
  }
  
});
