/******************************************************************************/
/*                 Start of User Data Sync Object Definition                  */
/******************************************************************************/

// JavaScript Object to hold Device information for webservice calls
var xMattersUserSync = BaseClass.extend({

  LOG_SOURCE: "xMattersUserSync.js: ",

  INVALID_USER: "INVALID_USER",
  INVALID_DEVICE: "INVALID_DEVICE",
  SUPERVISOR_INVALID: "SUPERVISOR_INVALID",
  UNKNOWN_SUPERVISOR: "UNKNOWN_SUPERVISOR",
 
  /**
   * Initialize the class variables
   */
  init: function(syncList) 
  {
    this.dataSyncHelper = new DataSyncHelper(syncList, "users");
    this.log = new Logger(this.LOG_SOURCE);
  },
   
  /**
   * Synchronize the user with xMatters
   * @param user the current user object
   */
  syncUser : function(user) 
  {
    try
    {
      if (!user.isValid()) 
      {
        throw this.INVALID_USER;
      }
      
      if (this.dataSyncHelper.checkIncludeExcludeList("User", user.targetName))
      {
        if (user.exists())
        {
          if (!USER_SEED_ONLY)
          {
            this.log.debug("Sending user update for " + user.targetName);
            this.handleSyncError(user.update(), user, "update");
          
            // Sync the devices
            this.syncDevices(user.targetName, user.devices);
          }
          else
          {
            this.log.info("Update for " + user.targetName + " will not be sent as USER_SEED_ONLY is " + USER_SEED_ONLY);
          }
        }
        else 
        {
          this.log.debug("Sending user add for " + user.targetName);
          this.handleSyncError(user.add(), user, "add");

          // Sync the devices
          this.syncDevices(user.targetName, user.devices);
        }
      } 
    }
    catch (e)
    {
      this.log.error("Failed to synchronize " + user.targetName + ": " + e);  
    }
  },
   
   /**
    * Synchronize the user devices with xMatters
    * @param owner user that owns this device
    * @param devices collection of devices belonging to the current user
    */
  syncDevices: function(owner, devices)
  {
    for (var i=0; i<devices.size(); i++) 
    {
      try
      {
        var device = devices.get(i);
        device.owner = owner;
        if (!device.isValid())
        {
          throw this.INVALID_DEVICE;
        }
        
        if (device.exists())
        {
          this.log.debug("Sending device update for " + device.deviceName);
          device.update();
        }
        else
        {
          this.log.debug("Sending device add for " + device.deviceName);
          device.add();
        }
      }
      catch (e)
      {
        this.log.error("Failed to synchronize " + device.deviceName + " for " + device.owner + ": " + e);  
      }
    }
  },

  /**
   * Function to deal with exceptions that require additional processing to get 
   * the object to sync to xMatters
   * @param response SOAP Response
   * @param user instance of the user object being synchronized
   * @param action the current action performed on the user
   */
  handleSyncError: function(response, user, action)
  {
    if (this.UNKNOWN_SUPERVISOR.equalsIgnoreCase(response) || this.SUPERVISOR_INVALID.equalsIgnoreCase(response))
    {
      var obj = new User();
      this.log.warn(user.targetName + ": SUPERVISOR_INVALID - [" + user.supervisors + "]. Retry using default supervisor [" + obj.supervisors + "]");
      user.supervisors = obj.supervisors;
      if ("add".equalsIgnoreCase(action))
      {
        user.add();
      }
      else
      {
        user.update();
      }
    }
  }
});